# Categorical models of state health policy institutional change
# Change in Concentration as response
# Chris Adolph
# May 2011

# Clear memory
rm(list=ls())

# Load libraries
library(lme4)
library(MASS)
library(simcf)
library(tile)
library(RColorBrewer)
library(verification)
source("setupCF.r")

sims <- 10000

# Load data
file <- "readydataSO3.csv"
data <- read.csv(file, header=TRUE)

# Model name
file <- "medicaidPlusM2"
main <- "Number of Agencies Lumped with Medicaid (M2: Add pop)"

# Set dependent variable
y <- "medicaidPlus"
ncat <- length(unique(na.omit(data[[y]])))
ystring <- paste("Medicaid +",0:(ncat-1))

# Set covariates
x <- c("unidem", "unirep",
       "rpincpc", "popdensity", "Unemployment",
       "povrate", "pctblack","pcthisp",
       "South", "Northeast", "West",
       "pctunder19", "pctover65",
       "RealPctChBudgetPC","PctHealthFed","PctWelfareFed",
       "trend",
       #"PublicHealthSL10","HospitalsSL10","MedicaidSL10",
       "logpop"
       )
# Create formula & get listwise deleted data
model <- as.formula(paste(y,"~", paste(c(x), collapse=" + "), sep=" "))
mdata <- extractdata(model, data, extra=~State+Year,na.rm=TRUE)
attach(mdata)
regVarnames <- c(y,x)
regData <- getxvar(regVarnames)

# Estimate appropriate model
yData <- mdata[[y]]
if (min(yData)<1) yData <- yData + (1-min(yData))
xData <- getxvar(x)
ls.result <- lm(yData~xData)            # use ls estimates as starting values
stval <- c(ls.result$coefficients,1:(ncat-2))  # initial guesses
oprobit.result <- optim(stval, llk.oprobit, method="BFGS", x=xData, y=yData, hessian=TRUE)
pe <- oprobit.result$par                # point estimates
vc <- solve(oprobit.result$hessian)     # var-cov matrix
se <- sqrt(diag(vc))                    # standard errors
ll <- -oprobit.result$value             # likelihood at maximum
aic <- 2*ncol(xData) - ll
bic <- log(nrow(xData))*ncol(xData) - ll
pcp <- pcp.oprobit(xData, yData, pe, constant=1, ncat=ncat, type="model")
pcpI <- pcp.oprobit(xData, yData, pe, constant=1, ncat=ncat, type="improve")
concord <- concord.oprobit(xData, yData, pe, constant=1, ncat=ncat, type="model")
concordI <- concord.oprobit(xData, yData, pe, constant=1, ncat=ncat, type="improve")

print(cbind(pe,se))
gofstring <- paste("AIC = ",round(aic,1),
                   "     BIC = ", round(bic,1),
                   "\n % Correctly Predicted = ", round(100*pcp,1),
                   "\n % Improvement on Null Model = ", round(100*pcpI,1),
                   "\n Concordance Index = ", round(concord,2),
                   "\n % Reduction in Discord = ", round(100*concordI,1),
                   sep="")

# Setup counterfactuals
cfscen <- setupCF(regData,regVarnames,model,fd=FALSE,mdata)
scenarios <- cfscen$scenarios
newdata <- cfscen$x
xpre <- cfscen$xpre
cfscenFD <- setupCF(regData,regVarnames,model,fd=TRUE,mdata)
scenariosFD <- cfscenFD$scenarios
newdataFD <- cfscenFD$x
xpreFD <- cfscenFD$xpre

# Simulate EV's and FD's
simbetas <- mvrnorm(sims,pe,vc)
simEy <- oprobitsimev(cfscen, simbetas, ci=0.95, constant=1, cat=ncat)
simFDy <- oprobitsimfd(cfscen, simbetas, ci=0.95, constant=1, cat=ncat)
simRRy <- oprobitsimrr(cfscen, simbetas, ci=0.95, constant=1, cat=ncat)

# Plot ropeladder(s)
# Plot ropeladder(s)
source("setupCF.R")
pch <- sigPch(simRRy)
baseline <- as.matrix(simEy$pe)[nrow(as.matrix(simEy$pe)),]
cfPlot(simEy, cfscen, fd=FALSE, baseline=baseline, limits=c(0.05,0.50), pch=pch,
       response=ystring, gof=gofstring, plotwidth=2, entryheight=0.12, output=file#,
       #main=main
       )
