# Categorical models of state health policy institutional change
# TEMPLATE
# Chris Adolph
# May 2011


# Clear memory
rm(list=ls())

# Load libraries
library(lme4)
library(MASS)
library(simcf)
library(tile)
library(RColorBrewer)
source("setupCF.r")

sims <- 10000

# Load data
file <- "readydataSO3.csv"
data <- read.csv(file, header=TRUE)

# Model name
file <- "mvphM2"
main <- "Combined Medicaid & Public Health (M2:  Add pop)"

# Set dependent variable
y <- "mvph"
ystring <- "Medicaid + Public Health agency"

# Set covariates
x <- c("unidem", "unirep",
       "rpincpc", "popdensity", "Unemployment",
       "povrate", "pctblack","pcthisp",
       "South", "Northeast", "West",
       "pctunder19", "pctover65",
       "RealPctChBudgetPC","PctHealthFed","PctWelfareFed",
       "trend",
       #"PublicHealthSL10","HospitalsSL10","MedicaidSL10",
       "logpop"
       )

# Create formula & get listwise deleted data
model <- as.formula(paste(y,"~", paste(c(x), collapse=" + "), sep=" "))
mdata <- extractdata(model, data, extra=~State+Year,na.rm=TRUE)
attach(mdata)
regVarnames <- c(y,x)
regData <- getxvar(regVarnames)

# Estimate appropriate model
logit.result <- glm(model, family=binomial, data=mdata)
pe <- logit.result$coefficients  # point estimates
vc <- vcov(logit.result)         # var-cov matrix
se <- sqrt(diag(vc))    # standard errors
aic <- AIC(logit.result)
bic <- AIC(logit.result, k = log(nrow(regData)))
pcp <- pcp.glm(logit.result, mdata[[y]], type="pcp")
pcpI <- pcp.glm(logit.result, mdata[[y]], type="improve")
concord <- concord.glm(logit.result, mdata[[y]], type="model")
concordI <- concord.glm(logit.result, mdata[[y]], type="improve")
print(summary(logit.result))
gofstring <- paste("AIC = ",round(aic,1),
                   "     BIC = ", round(bic,1),
                   "\n % Correctly Predicted = ", round(100*pcp,1),
                   "\n % Improvement on Null Model = ", round(100*pcpI,1),
                   "\n Concordance Index = ", round(concord,2),
                   "\n % Reduction in Discord = ", round(100*concordI,1),
                   sep="")

# Setup counterfactuals
cfscen <- setupCF(regData,regVarnames,model,fd=FALSE,mdata)
scenarios <- cfscen$scenarios
newdata <- cfscen$x
xpre <- cfscen$xpre
cfscenFD <- setupCF(regData,regVarnames,model,fd=TRUE,mdata)
scenariosFD <- cfscenFD$scenarios
newdataFD <- cfscenFD$x
xpreFD <- cfscenFD$xpre

# Simulate EV's and FD's
simbetas <- mvrnorm(sims,pe,vc)
simEy <- logitsimev(cfscen, simbetas, ci=0.95, constant=1)
simFDy <- logitsimfd(cfscen, simbetas, ci=0.95, constant=1)
simRRy <- logitsimrr(cfscen, simbetas, ci=0.95, constant=1)

# Check plot limits
print(c(min(unlist(simEy)), max(unlist(simEy))))

# Plot ropeladder(s)
source("setupCF.R")
pch <- sigPch(simRRy)
cfPlot(simEy, cfscen, fd=FALSE, baseline=simEy$pe[length(simEy$pe)], limits=c(0.04,0.85), pch=pch,
       response=ystring, gof=gofstring, entryheight=0.08, output=file#,
       #main=main
       )
